module Grays42_GPath

	class GcodeToGeom
		
		def initialize(text)
			@text = text
			@snippet_stack = []
			@current_point = [0,0,3]
			
			grab_all_gcode
			begin_drawing_gcode
			
			return nil
		end
		
		GcodeSnippet = Struct.new(:code,:x,:y,:z,:i,:j,:k)
		
		def grab_all_gcode
			all_captures = @text.scan(/((G\d+).+?\n)/)
			
			all_captures.each do |cap|
				next if cap.nil? or cap.length < 2
				snippet = GcodeSnippet.new
				snippet.code = cap[1].to_sym
				coords = cap[0].scan(/([A-Z])([\-\d\.]+)/)
				coords.each do |c|
					s = c[0].downcase
					val = c[1].to_f
					snippet[s.to_sym] = val if snippet.members.include?(s)
				end
				@snippet_stack << snippet
			end
			
		end
		
		def begin_drawing_gcode
			Sketchup.active_model.start_operation("Draw toolpath from gcode")
			group = Sketchup.active_model.entities.add_group
			entities = group.entities
			
			@time = 0.05
			@iter = 10
			0.upto(@snippet_stack.length/@iter) do |i|#@snippet_stack.length - 1) do |i|
				UI.start_timer(i*@time,false) do
					0.upto(@iter-1) do |j|
						real_i = i*@iter + j
						draw_gcode(@snippet_stack[(i*@iter)+j],entities) unless real_i >= @snippet_stack.length
					end
				end
			end
			UI.start_timer(@snippet_stack.length,false) {Sketchup.active_model.commit_operation}
		end
		
		def draw_gcode(snippet,entities)
				straight = [:G00,:G0,:G01,:G1]
				arc = [:G02,:G2,:G03,:G3]
				if straight.include?(snippet.code) then
					draw_line(snippet,entities)
				elsif arc.include?(snippet.code) then
					draw_arc(snippet,entities)
				elsif snippet.code == :G04 then
					dwell(snippet,entities)
				end
		end
		
		def draw_line(snippet,entities)
			p = get_point(snippet)
			entities.add_line(@current_point,p)
			@current_point = p
		end
		
		def draw_arc(snippet,entities)
			end_point = get_point(snippet)
			@current_point.z = end_point.z
			center = @current_point.clone
			center.x = center.x + snippet.i
			center.y = center.y + snippet.j
			start_vector = center.vector_to(@current_point.clone).normalize
			end_vector = center.vector_to(end_point.clone).normalize
			normal = Geom::Vector3d.new(0,0,1)
			normal.reverse! if snippet.code == :G2 or snippet.code == :G02
			radius = center.distance(end_point)
			@current_point = end_point
			
			PGL_ArcDef.new(center,radius,start_vector,end_vector,normal).draw(entities)
		end
		
		def get_point(snippet)
			snippet.x = @current_point.x unless snippet.x
			snippet.y = @current_point.y unless snippet.y
			snippet.z = @current_point.z unless snippet.z
			return Geom::Point3d.new(snippet.x,snippet.y,snippet.z)
		end
		
		def dwell(snippet,entities)
		
		end
	end
	
end