module Grays42_GPath

	class Job
		
		#These are used to decide which piece to go to when optimizing the path for seeking (because you might want to cut biggest
		#pieces last rather than just use the shortest possible path). Changing these values changes the way that path selection
		#is weighted.
		OPTIMIZE_PIECE_LENGTH_SCALAR = 0.5
		OPTIMIZE_PIECE_DISTANCE_SCALAR = 0.5
		
		def initialize(environment)
			@env = environment
			@workpieces = []
			@screws = []
		end
		
		attr_accessor :workpieces
		attr_accessor :screws
		attr_accessor :env
		
		#Extremely rough 
		def optimize_workpieces
			new_workpiece_list = []
			
			longest_path = @workpieces[0].length
			@workpieces.each do |wp|
				longest_path = [longest_path,wp.length].max
			end
			
			
			failsafe = 1000
			while @workpieces.length > 0 do
				failsafe = failsafe - 1
				raise "Error: infinite loop" if failsafe == 0
				
				start = Geom::Point3d.new(@env.home_x,@env.home_y,@env.home_z) if new_workpiece_list.length == 0
				start = new_workpiece_list.last.end_pt if new_workpiece_list.length > 0
				
				longest_distance = nil
				longest_length = nil
				@workpieces.each do |wp|
					dist = start.distance(wp.start_pt)
					if longest_distance.class == NilClass or dist < longest_distance then
						longest_distance = dist
					end
					len = wp.length
					if longest_length.class == NilClass or len < longest_length then
						longest_length = len
					end
				end
				
				selection_weight = nil
				selected_workpiece = nil
				@workpieces.each do |workpiece|
					this_weight = start.distance(workpiece.start_pt) * OPTIMIZE_PIECE_DISTANCE_SCALAR / longest_distance
					this_weight += workpiece.length * OPTIMIZE_PIECE_LENGTH_SCALAR / longest_length
					if selected_workpiece.class == NilClass or this_weight < selection_weight then
						selected_workpiece = workpiece
						selection_weight = this_weight
					end
				end
				
				@workpieces.delete(selected_workpiece)
				new_workpiece_list << selected_workpiece
			end
			@workpieces = new_workpiece_list
		end
		
		def optimize_screws
			new_screws = []
			
			failsafe = 1000
			while @screws.length > 0 do
				failsafe = failsafe - 1
				raise "Error: infinite loop" if failsafe == 0
				
				start = Geom::Point3d.new(@env.home_x,@env.home_y,@env.home_z) if new_screws.length == 0
				start = new_screws.last if new_screws.length > 0
				
				distance = nil
				shortest_screw = nil
				@screws.each do |screw_pt|
					this_distance = start.distance(screw_pt)
					if distance.class == NilClass or this_distance < distance then
						shortest_screw = screw_pt
						distance = this_distance
					end
				end
				
				@screws.delete(shortest_screw)
				new_screws << shortest_screw
			end
			
			@screws = new_screws
		end
		
		def add_faced_group(group)
			@workpieces << Solid_Paths.new(group)
		end
		
		def add_unfaced_group(group)
			
		end
		
		def add_screw(group)
			@screws << group.bounds.center
		end
		
		def add_group(group)
			raise "Class error: #{group.class}" unless group.class == Sketchup::ComponentInstance or group.class == Sketchup::Group
			
			definition = group if group.class == Sketchup::Group
			definition = group.definition if group.class == Sketchup::ComponentInstance
			
			num_faces = 0
			definition.entities.each do |e|
				num_faces += 1 if e.class == Sketchup::Face
			end
			
			if group.name == "SCREW" or definition.name == "SCREW" then
				add_screw(group)
			elsif num_faces == 0 then
				add_unfaced_group(group)
			elsif num_faces > 0 then
				add_faced_group(group)
			end
		end
		
		def offset_sets
			@workpieces.each do |workpiece|
				workpiece.paths.each do |path|
					path.offset!(@env.tool_radius + @env.interior_radius)
					path.offset!(-@env.interior_radius)
				end
			end
		end
		
		def draw_workpieces
			@workpieces.each do |workpiece|
				workpiece.draw
			end
		end
		
		def get_gcode
			optimize_workpieces
			gen = Gcode_Generator.new(@env)
			
			gen.header
			@workpieces.each do |workpiece|
				gen.cut_group(workpiece)
			end
			gen.footer
			
			return gen.gcode
		end
		
		def get_screw_gcode
			return nil if @screws.length == 0
			optimize_screws
			
			gen = Gcode_Generator.new(@env)
			
			gen.header
			gen.initialize_group
			@screws.each do |screw|
				gen.cut_screw(screw)
			end
			gen.close_group
			gen.footer
			
			return gen.gcode
		end
		
		def draw_job
			gcode = get_gcode
			GcodeToGeom.new(gcode)
		end
		
		def draw_screws
			gcode = get_screw_gcode
			GcodeToGeom.new(gcode)
		end
	end
end