module Grays42_GPath

	if not $gpath_ui_loaded then
		$gpath_ui_submenu = UI.menu("Tools").add_submenu("GPath")
		$gpath_ui_loaded = true
	end

	if not $gpath_ui_console_loaded then
		environment_prompt_cmd = UI::Command.new("Set Environment") { UI_Handler::env_prompt }
		environment_prompt_cmd.tooltip ="Set Environment"
		environment_prompt_cmd.status_bar_text = "Set Variables for CNC Environment"
		environment_prompt_cmd.menu_text = "Set Environment"
		$gpath_ui_submenu.add_item(environment_prompt_cmd)
		
		environment_view_cmd = UI::Command.new("View Environment") { UI_Handler::env_view }
		environment_view_cmd.tooltip ="View Environment"
		environment_view_cmd.status_bar_text = "View Variables for CNC Environment"
		environment_view_cmd.menu_text = "View Environment"
		$gpath_ui_submenu.add_item(environment_view_cmd)

		draw_toolpath_cmd = UI::Command.new("View Toolpath") { UI_Handler::draw_toolpath(Sketchup.active_model.selection) }
		draw_toolpath_cmd.tooltip = "View Toolpath"
		draw_toolpath_cmd.status_bar_text = "Draw toolpath for selected items"
		draw_toolpath_cmd.menu_text = "View Toolpath"
		$gpath_ui_submenu.add_item(draw_toolpath_cmd)

		draw_screws_cmd = UI::Command.new("View Screws") { UI_Handler::draw_screws(Sketchup.active_model.selection) }
		draw_screws_cmd.tooltip = "View Screws"
		draw_screws_cmd.status_bar_text = "View any screws for selected items"
		draw_screws_cmd.menu_text = "View Screws"
		$gpath_ui_submenu.add_item(draw_screws_cmd)

		save_toolpath_cmd = UI::Command.new("Save Toolpath") { UI_Handler::save_toolpath(Sketchup.active_model.selection) }
		save_toolpath_cmd.tooltip = "Save Toolpath"
		save_toolpath_cmd.status_bar_text = "Save toolpath for selected items"
		save_toolpath_cmd.menu_text = "Save Toolpath"
		$gpath_ui_submenu.add_item(save_toolpath_cmd)
		
		$gpath_ui_console_loaded = true
	
	end

	class UI_Handler
		
		FLOAT_REGEX = /[\d\.]+/
		DICT_STRING = "Grays42_GPath"
		SCRIPT_FILENAME_STRING = "script_filename"

		def self.default_env
			#Load default environment
			env1 = Environment_Definition.new
			env1.tool_radius = (1.0/8.0).to_l
			env1.interior_radius = (1.0/8.0).to_l
			env1.cut_z_array = [0.35.to_l,0.2.to_l,0.05.to_l]
			env1.gradual_descent = true
			env1.seek_rate = 50.to_l
			env1.feed_rate = 30.to_l
			env1.vertical_feed_rate = 6.to_l
			env1.safe_height = 2.to_l
			env1.material_clearance = 0.5.to_l
			env1.material_clearance = 0.35.to_l
			env1.dwell_time = 0.25
			env1.home_x = 0.to_l
			env1.home_y = 0.to_l
			env1.home_z = 2.to_l
			
			return env1
		end
		
		def self.reset_env
			set_env(default_env)
		end
		
		def self.set_env(env1)
			Sketchup.active_model.set_attribute(DICT_STRING,"env",env1.instance_variables_hash)
			return true
		end
		
		def self.get_env(failsafe = 0)
			failsafe = failsafe + 1
			raise "Error: infinite loop" if failsafe == 3
			stored_env = Sketchup.active_model.get_attribute(DICT_STRING,"env")
			if Environment_Definition::valid_stored_env?(stored_env)
				env1 = Environment_Definition.new
				env1.unpack_variables_hash(stored_env)
				return env1
			else
				UI.messagebox("Error: could not properly store default environment.\n\n#{stored_env.to_s}") if failsafe == 1
				UI.messagebox("CNC Environment missing or invalid. Loading default environment.") if failsafe == 0
				reset_env
				return get_env(failsafe)
			end
		end
		
		def self.env_view
			UI.messagebox(get_env.pretty_string)
		end
		
		def self.env_prompt
			env1 = get_env
			prompt_pairs = []
			
			def self.arr_to_str(arr)
				str = ""
				arr.each do |v|
					str << "#{v.to_s},"
				end
				return str.chop
			end
			
			prompt_pairs << ["Tool Radius (in)",env1.tool_radius.to_s]
			prompt_pairs << ["Interior Radius (in)", env1.interior_radius.to_s]
			prompt_pairs << ["Cut Layers (in,in,...)",arr_to_str(env1.cut_z_array)]
			prompt_pairs << ["Descend gradually? (true/false)",env1.gradual_descent.to_s]
			prompt_pairs << ["Seek Rate (in/min)",env1.seek_rate.to_s]
			prompt_pairs << ["Feed Rate (in/min)",env1.feed_rate.to_s]
			prompt_pairs << ["Max Vertical Feed Rate (in/min)",env1.vertical_feed_rate.to_s]
			prompt_pairs << ["Dwell Time (s)",env1.dwell_time.to_s]
			prompt_pairs << ["Safe Seek Height (in)",env1.safe_height.to_s]
			prompt_pairs << ["Material Clearance (in)",env1.material_clearance.to_s]
			prompt_pairs << ["Screw Depth (in)",env1.material_clearance.to_s]
			prompt_pairs << ["Home Position (x,y,z)","#{env1.home_x.to_f}, #{env1.home_y.to_f}, #{env1.home_z.to_f}"]
			
			prompts = []
			defaults = []
			prompt_pairs.each do |pair|
				prompts << pair[0]
				defaults << pair[1]
			end
			
			input = UI.inputbox(prompts,defaults,"","Environment Definition")
			UI.messagebox("No changes made to environment definition.") if input == false
			return if input == false
			
			env2 = Environment_Definition.new
			env2.tool_radius = input[0].to_l
			env2.interior_radius = input[1].to_l
			env2.cut_z_array = []
			input[2].scan(FLOAT_REGEX).each do |v|
				env2.cut_z_array << v.to_l
			end
			env2.gradual_descent = false
			env2.gradual_descent = true if input[3].downcase.match("true")
			env2.seek_rate = input[4].to_l
			env2.feed_rate = input[5].to_l
			env2.vertical_feed_rate = input[6].to_l
			env2.dwell_time = input[7].to_f
			env2.safe_height = input[8].to_l
			env2.material_clearance = input[9].to_l
			env2.screw_depth = input[10].to_l
			h = input[11].scan(FLOAT_REGEX)
			if h.length == 3 then
				env2.home_x = h[0].to_l
				env2.home_y = h[1].to_l
				env2.home_z = h[2].to_l
				
			else
				env2.home_x = default_env.home_x
				env2.home_y = default_env.home_y
				env2.home_z = default_env.home_z
			end
			
			str = "Please verify new environment settings:\n\n#{env2.pretty_string}"
			verify = UI.messagebox(str,MB_OKCANCEL)
			
			UI.messagebox("Environment definition saved for this model.") if verify == IDOK
			set_env(env2) if verify == IDOK
			env_prompt if verify == IDCANCEL
			return
		end
		
		def self.save_toolpath(selection)
			model_dir = File.dirname(Sketchup.active_model.path)
			root_name = File.basename(Sketchup.active_model.path,File.extname(Sketchup.active_model.path))
			filename = Sketchup.active_model.get_attribute(DICT_STRING,SCRIPT_FILENAME_STRING)
			filename = "#{root_name}.cnc" if filename.nil?
			screw_filename = "#{root_name}_screws.cnc"
			full_path = File.expand_path(filename,model_dir)
			screw_full_path = File.expand_path(screw_filename,model_dir)
			
			input = UI.inputbox(["Save as:"],["#{root_name}.cnc"])
			return if input == false
			
			job = Job.new(get_env)
			selection.each do |s|
				job.add_group(s)
			end
			job.offset_sets
			
			file = File.open(full_path, 'w')
			file.write(job.get_gcode)
			file.close_write
			file_saved_string = "Code has been saved to:\n#{full_path}"
			
			screw_code = job.get_screw_gcode
			if screw_code then
				file2 = File.open(screw_full_path, 'w')
				file2.write(screw_code)
				file2.close_write
				file_saved_string << "\n\nScrew code has been saved to:\n#{screw_full_path}"
			end
			
			UI.messagebox(file_saved_string) if file.closed? == true
			UI.messagebox("Error: could not save to\n\n#{full_path}") if file.closed? == false
			Sketchup.active_model.set_attribute(DICT_STRING,SCRIPT_FILENAME_STRING,filename)
		end

		def self.draw_toolpath(selection)
			job = Job.new(get_env)
			selection.each do |s|
				job.add_group(s)
			end
			job.offset_sets
			job.draw_job
		end
		
		def self.draw_screws(selection)
			job = Job.new(get_env)
			selection.each do |s|
				job.add_group(s)
			end
			
			if job.screws.length == 0 then
				UI.messagebox("Error: no screws to draw. To make a screw, create a group or instance and name it \"SCREW\" with the center point being the location you want the screw to be marked.")
			end
			
			job.draw_screws
		end
	end
end