load 'gpath/main.rb'
include Grays42_GPath
require 'test/unit'

$debug = false

class ENTITY_TESTS < Test::Unit::TestCase

	def ents
		return Sketchup.active_model.entities
	end
	
	def teardown
		$debug = false
	end
	
	def norm
		return Geom::Vector3d.new(0,0,1)
	end
	
	def refresh
		Sketchup.active_model.active_view.refresh
	end
	
	def erase(ent)
		if ent.class == Array then
			ents.erase_entities(ent)
		else
			ents.erase_entities([ent])
		end
	end

	def pt(x,y)
		return Geom::Point3d.new(x,y,0)
	end
	
	def vec(x,y)
		return Geom::Vector3d.new(x,y,0)
	end
	
	def seg(x1,y1,x2,y2)
		return PGL_SegmentDef.new(pt(x1,y1),pt(x2,y2))
	end
	
	def to_rad(num)
		return num.to_f*Math::PI/180.0
	end

	def test_basic_edge
		
		edge1 = ents.add_line(pt(0,0),pt(0,1))
		path = Entity_Path.new
		path << edge1
		
		assert_equal(1,path.length)
		assert_equal(1,path.array_length)
		assert_equal(pt(0,0),path.start_pt)
		assert_equal(pt(0,1),path.end_pt)
		
		erase(edge1)
	end
	
	def test_basic_arc
		path_a = Entity_Path.new
		a = ents.add_arc(pt(2,2),vec(1,0),norm,2,0,90.degrees)
		path_a << a
		erase(a)
		
		path_b = Entity_Path.new
		b = PGL_ArcDef.new(pt(2,2), 2, vec(1,0), vec(0,1))
		path_b << b
	end
	
	def test_basic_path
		a = ents.add_arc(pt(2,2),vec(1,0),norm,2,0,90.degrees)
		b = ents.add_line(pt(2,4),pt(0,1))
		entity_path = Entity_Path.new
		entity_path << a
		entity_path << b
		erase(a)
		erase(b)
		refresh
		
		pgl_path = PGL_Path.new
		pgl_path << PGL_ArcDef.new(pt(2,2), 2, vec(1,0),vec(0,1))
		pgl_path << PGL_SegmentDef.new(pt(2,4),pt(0,1))
		
		assert_equal(pgl_path,entity_path)
		
		pgl_path.offset!(1)
		entity_path.offset!(1)
		assert_equal(pgl_path,entity_path)
	end
	
	def test_basic_circle
		a = ents.add_arc(pt(3,2),vec(1,0),norm,2,0.degrees,90.degrees)
		b = ents.add_arc(pt(3,2),vec(1,0),norm,2,90.degrees,360.degrees)
		entity_path = Entity_Path.new
		entity_path << a
		entity_path << b
		entity_path.close!
		erase(a)
		erase(b)
		refresh
		
		assert_equal(1,entity_path.array_length)
		assert_equal(360.0,entity_path.last.sweep*180/Math::PI)
		assert_equal(entity_path.start_pt,entity_path.end_pt)
	end
	
	def test_basic_interior_corner
		rad = 3.0*(Math::sqrt(2.0))/8.0
		corner_center = pt(-1 - rad, -2 + rad)
		a = ents.add_line(pt(-1,0),pt(-1,-0.8))
		b = ents.add_arc(corner_center,vec(1,1),norm.reverse,3.0/4.0,0.degrees,195.degrees)
		c = ents.add_line(pt(-2.1,-2),pt(-3,-2))
		
		entity_path = Entity_Path.new
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		entity_path << c
		erase(c)
		refresh
		
		entity_path.set_array_precise
		entity_path.offset!(-0.75)
				
		pgl_path = PGL_Path.new
		pgl_path << seg(-1.75,0,-1.75,-2+rad*2)
		c1 = pt(-1,-2+rad*2)
		pgl_path << PGL_ArcDef.new(c1,0.75,c1.vector_to(pgl_path.last.end_pt),c1.vector_to(corner_center))
		c2 = pt(-2*rad - 1,-2)
		pgl_path << PGL_ArcDef.new(c2,0.75,c2.vector_to(pgl_path.last.end_pt),vec(0,1))
		pgl_path << PGL_SegmentDef.new(pgl_path.last.end_pt,pt(-3,-1.25))
		
		assert_equal(pgl_path,entity_path)
		
	end
	
	def test_advanced_interior_corner

		rad = 3.0*(Math::sqrt(2.0))/8.0
		corner_center = pt(2 - rad, -2 + rad)
		a = ents.add_line(pt(2,0),pt(2,-1*(2 - 0.75*Math::sqrt(2))))
		b = ents.add_arc(corner_center,vec(1,1),norm.reverse,3.0/4.0,0.degrees,180.degrees)
		c = ents.add_line(pt(2 - 0.75*Math::sqrt(2),-2),pt(0,-2))
		d = ents.add_line(pt(0,-2),pt(0,0))
		e = ents.add_line(pt(0,0),pt(2,0))
		
		entity_path = Entity_Path.new
		entity_path << b
		erase(b)
		entity_path << c
		erase(c)
		entity_path << d
		erase(d)
		entity_path << e
		erase(e)
		entity_path << a
		erase(a)
		refresh
		entity_path.close!
		
		entity_path.set_array_precise
		entity_path.offset!(-0.75)
	
		pgl_path = PGL_Path.new
		c2 = pt(-2*rad + 2,-2)
		pgl_path << PGL_ArcDef.new(c2,0.75,vec(1,1),vec(0,1))
		pgl_path << PGL_SegmentDef.new(pgl_path.last.end_pt,pt(0.75,-1.25))
		pgl_path << PGL_SegmentDef.new(pgl_path.last.end_pt,pt(0.75,-0.75))
		pgl_path << PGL_SegmentDef.new(pgl_path.last.end_pt,pt(1.25,-0.75))
		pgl_path << PGL_SegmentDef.new(pgl_path.last.end_pt,pt(1.25,-1*(2 - 0.75*Math::sqrt(2))))
		c1 = pt(2,-2+rad*2)
		pgl_path << PGL_ArcDef.new(c1,0.75,c1.vector_to(pgl_path.last.end_pt),c1.vector_to(corner_center))
		assert_equal(pgl_path,entity_path)
		
	end
	
	def test_basic_square_face
		face = ents.add_face(pt(2,2),pt(2,4),pt(4,4),pt(4,2))
		loop = face.loops[0]
		entity_path = Entity_Path.new
		loop.edges.each do |e|
			entity_path << e
		end
		entity_path.offset!(1)
		erase(loop.edges)
	end
	
	def test_nonseq_joins_first_two
		pgl_path = PGL_Path.new
		pgl_path << seg(0,0,0,1)
		pgl_path << seg(0,1,1,1)
		
		entity_path = Entity_Path.new
		a = ents.add_line(pt(0,0),pt(0,1))
		b = ents.add_line(pt(1,1),pt(0,1))
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		refresh
		assert_equal(pgl_path,entity_path)
		
		entity_path = Entity_Path.new
		a = ents.add_line(pt(0,1),pt(0,0))
		b = ents.add_line(pt(1,1),pt(0,1))
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		refresh
		assert_equal(pgl_path,entity_path)
		
		entity_path = Entity_Path.new
		a = ents.add_line(pt(0,1),pt(0,0))
		b = ents.add_line(pt(0,1),pt(1,1))
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		refresh
		assert_equal(pgl_path,entity_path)
		
		entity_path = Entity_Path.new
		a = ents.add_line(pt(0,1),pt(0,0))
		b = ents.add_line(pt(1,1),pt(0,1))
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		refresh
		assert_equal(pgl_path,entity_path)
		
	end
	
	def test_nonseq
		pgl_path = PGL_Path.new
		pgl_path << seg(0,0,0,1)
		pgl_path << seg(0,1,1,1)
		pgl_path << seg(1,1,1,2)
		pgl_path << seg(1,2,2,2)
		
		entity_path = Entity_Path.new
		a = ents.add_line(pt(0,1),pt(0,0))
		b = ents.add_line(pt(1,1),pt(0,1))
		c = ents.add_line(pt(1,2),pt(1,1))
		d = ents.add_line(pt(2,2),pt(1,2))
		entity_path << a
		erase(a)
		entity_path << b
		erase(b)
		entity_path << c
		erase(c)
		entity_path << d
		erase(d)
		refresh
		assert_equal(pgl_path,entity_path)
		
	end
	
	def test_reversed_arcs
	end
	
	def test_full_circle
		forward_circle = ents.add_arc(pt(2,2),vec(1,0),norm,2,0.degrees,360.degrees)
		ep1 = Entity_Path.new
		ep1 << forward_circle
		erase(forward_circle)
		refresh
		assert_equal(ep1.offset(1).pgl_items[0],PGL_ArcDef.new(pt(2,2),1,vec(1,0),vec(1,0),norm))
		
		backward_circle = ents.add_arc(pt(2,2),vec(1,0),norm.reverse,2,360.degrees,0.degrees)
		ep2 = Entity_Path.new
		ep2 << backward_circle
		erase(backward_circle)
		refresh
		assert_equal(ep2.offset(1).pgl_items[0],PGL_ArcDef.new(pt(2,2),3,vec(1,0),vec(1,0),norm.reverse))
	end
end
