load 'gpath/main.rb'
include Grays42_GPath
require 'test/unit'

$debug = false


## NOTE: THIS UNIT TEST SUITE WILL ONLY CORRECTLY FUNCTION ON THE TEST BED MODEL.

## DISABLE THIS UNIT TEST SUITE IF TESTING UNDER A DIFFERENT FILE.

class FACE_TESTS < Test::Unit::TestCase

	def ents
		return Sketchup.active_model.entities
	end
	
	def teardown
		$debug = false
		$debug2 = false
	end
	
	def norm
		return Geom::Vector3d.new(0,0,1)
	end
	
	def refresh
		Sketchup.active_model.active_view.refresh
	end
	
	def erase(ent)
		if ent.class == Array then
			ents.erase_entities(ent)
		else
			ents.erase_entities([ent])
		end
	end

	def pt(x,y)
		return Geom::Point3d.new(x,y,0)
	end
	
	def vec(x,y)
		return Geom::Vector3d.new(x,y,0)
	end
	
	def seg(x1,y1,x2,y2)
		return PGL_SegmentDef.new(pt(x1,y1),pt(x2,y2))
	end
	
	def to_rad(num)
		return num.to_f*Math::PI/180.0
	end
	
	def get_group(name)
		ents.each do |e|
			next unless e.methods.include?("name")
			return e if e.name == name
		end
		return nil
	end
	
	def test_face_capture
		paths = get_paths_from_group(get_group("F1"))
		
		path1 = PGL_Path.new
		path1 << seg(8,1,6,1)
		path1 << seg(6,1,6,3)
		path1 << seg(6,3,8,3)
		path1 << seg(8,3,8,1)
		assert_equal(path1,paths[0])
		
		path2 = PGL_Path.new
		path2 << seg(6.125,2.875,6.125,1.125)
		path2 << seg(6.125,1.125,7.875,1.125)
		path2 << seg(7.875,1.125,7.875,2.875)
		path2 << seg(7.875,2.875,6.125,2.875)
		assert_equal(path2,paths[1])
	end
	
	def test_get_bottom_face

		solid = Solid_Paths.new(get_group("G3"))
		assert_equal(1,solid.paths.length)
		
		paths = []
		["G1","G2", "G3"].each do |s|
			solid = Solid_Paths.new(get_group(s))
			paths = paths + solid.paths
		end
		
		#Note: had to arbitrarily redo the order for these due to the new TSP routine, if TSP changes
		#again these tests may break, just need to shuffle the paths around a bit.
		
		p0 = PGL_Path.new
		p0 << seg(8,6,8,4)
		p0 << seg(8,4,6,4)
		p0 << seg(6,4,6,6)
		p0 << seg(6,6,8,6)
		assert_equal(p0,paths[0])
		
		p1 = PGL_Path.new
		p1 << seg(9,6,11,6)
		p1 << seg(11,6,11,4)
		p1 << seg(11,4,9,4)
		p1 << seg(9,4,9,6)
		assert_equal(p1,paths[2])
		
		assert_equal(1,paths[1].array_length)
		assert_equal(Geom::Point3d.new(10,5,0),paths[1].last.center)
		assert_equal(0.5,paths[1].last.radius)
		
		p3 = PGL_Path.new
		p3 << seg(13.5,6,13.5,4)
		p3 << seg(13.5,4,12.5,4)
		p3 << seg(12.5,4,12.5,6)
		p3 << seg(12.5,6,13.5,6)
		assert_equal(p3,paths[3])
		
	end
	
	def test_components
		solid = Solid_Paths.new(get_group("C2a"))
		assert_equal(1,solid.paths.length)
		
		paths = []
		["C2a","C2b", "C2c"].each do |s|
			solid = Solid_Paths.new(get_group(s))
			paths = paths + solid.paths
		end
		
		#Note: had to arbitrarily redo the order for these due to the new TSP routine, if TSP changes
		#again these tests may break, just need to shuffle the paths around a bit.
		
		p0 = PGL_Path.new
		p0 << seg(8,6,8,4)
		p0 << seg(8,4,6,4)
		p0 << seg(6,4,6,6)
		p0 << seg(6,6,8,6)
		p0.transform!(Geom::Transformation.new(Geom::Vector3d.new(0,3,0)))
		assert_equal(p0,paths[0])
		
		p1 = PGL_Path.new
		p1 << seg(9,6,11,6)
		p1 << seg(11,6,11,4)
		p1 << seg(11,4,9,4)
		p1 << seg(9,4,9,6)
		p1.transform!(Geom::Transformation.new(Geom::Vector3d.new(0,3,0)))
		assert_equal(p1,paths[2])
		
		assert_equal(1,paths[1].array_length)
		assert_equal(Geom::Point3d.new(10,8,0),paths[1].last.center)
		assert_equal(0.5,paths[1].last.radius)
		
		p3 = PGL_Path.new
		p3 << seg(13.5,6,13.5,4)
		p3 << seg(13.5,4,12.5,4)
		p3 << seg(12.5,4,12.5,6)
		p3 << seg(12.5,6,13.5,6)
		p3.transform!(Geom::Transformation.new(Geom::Vector3d.new(0,3,0)))
		assert_equal(p3,paths[3])
	end
	
	def get_paths_from_group(g)
		paths = []
		g.entities.each do |e|
			next unless e.class == Sketchup::Face
			e.loops.each do |loop|
				edges = loop.edges
				path = Entity_Path.new
				edges.each do |d|
					path << d
				end

				#explicitly check circles.
				if path.circle? then
					path.reverse! if path.pgl_items[0].direction == :counter_clockwise and loop.outer?
					path.reverse! if path.pgl_items[0].direction == :clockwise and not loop.outer?
				end
				
				path.close!
				path.transform!(g.transformation)
				paths << path
			end
		end
		return paths
	end
	
	def test_first_face
		solid = Solid_Paths.new(get_group("E1a"))
		solid.offset!(6.0/32.0)
		solid.offset!(-3.0/32.0)
	end
	
	def test_faces
		groups = {}
		ents.each do |e|
			next unless e.methods.include?("name")
			face_match = e.name.match(/^F\d+$/)
			if e.class == Sketchup::Group and face_match then
				groups[e.name] = e
			end
		end
		groups.keys.sort!
		
		1.upto(20) do |i|
			name = "F#{i}"
			next unless groups.keys.include?(name)
			paths = get_paths_from_group(groups[name])
			paths.each do |p|
				p.offset!(0.125)
				p.offset!(-0.0675)
			end
		end
		
		
		return
		ents.each do |e|
			match = e.name.match(/^(F\d+)_([\d\.])+$/)
			if e.class == Sketchup::Group and match then
				base_name = match[1]
				offset_amount = match[2].to_f
				offsets[base_name] = [] unless offsets.keys.include?(base_name)
			end
		end
	end
	
end
