load 'gpath/main.rb'
include Grays42_GPath
require 'test/unit'

$debug = false
$master_debug_var = 0
$debug_num = nil

# IMPORTANT NOTE: Many of the tests in this section must be verified VISUALLY because there's way too much data to input test cases for everything.
# Some tests may PASS despite being invalid. Check manually!

class JOB_TESTS < Test::Unit::TestCase
	
	def visual_test
		#FLAG TRUE FOR VISUAL TEST
		return true
	end

	def ents
		return Sketchup.active_model.entities
	end

	def pt(x,y)
		return Geom::Point3d.new(x,y,0)
	end
	
	def vec(x,y)
		return Geom::Vector3d.new(x,y,0)
	end
	
	def seg(x1,y1,x2,y2)
		return PGL_SegmentDef.new(pt(x1,y1),pt(x2,y2))
	end
	
	def to_rad(num)
		return num.to_f*Math::PI/180.0
	end
	
	def teardown
		$debug = false
	end

	def get_group(name)
		ents.each do |e|
			next unless e.methods.include?("name")
			return e if e.name == name
		end
		return nil
	end
	
	def ex_job
		job = Job.new(env)
		job.add_group(get_group("G1"))
		return job
		
	end

	def ex_path
		p0 = PGL_Path.new
		p0 << seg(8,6,8,4)
		p0 << seg(8,4,6,4)
		p0 << seg(6,4,6,6)
		p0 << seg(6,6,8,6)
		return p0
	end

	def env
		#Load default environment
		env1 = Environment_Definition.new
		env1.tool_radius = 0.125
		env1.interior_radius = 0.125
		env1.cut_z_array = [0.35,0.2,0.05]
		env1.gradual_descent = true
		env1.feed_rate = 18
		env1.seek_rate = 30
		env1.vertical_feed_rate = 3
		env1.safe_height = 2
		env1.material_clearance = 0.51
		env1.dwell_time = 0.25
		env1.home_position = Geom::Point3d.new(0,28,3)
		
		env1.workspace = Geom::BoundingBox.new
		env1.workspace.add(Geom::Point3d.new(0,0,0))
		env1.workspace.add(Geom::Point3d.new(52, 28, 2.22))
		
		env1.validate!
		
		return env1
	end
	
	def test_add_group
		job = Job.new(env)
		job.add_group(get_group("G1"))
		
		p0 = PGL_Path.new
		p0 << seg(8,6,8,4)
		p0 << seg(8,4,6,4)
		p0 << seg(6,4,6,6)
		p0 << seg(6,6,8,6)
		
		assert_equal(p0,job.workpieces[0].paths[0])
	end
	
	def test_visual_confirm
		env1 = env
		env1.tool_radius = 3.0/32.0
		env1.interior_radius = 3.0/32.0
		job = Job.new(env1)
		job.add_group(get_group("E1a"))
		job.offset_sets
		#job.draw_job
		
	end
	
	def test_offset_group
		env1 = env
		job = Job.new(env1)
		job.add_group(get_group("G1"))
		job.offset_sets
		
		p0 = PGL_Path.new
		p0 << seg(8,6,8,4)
		p0 << seg(8,4,6,4)
		p0 << seg(6,4,6,6)
		p0 << seg(6,6,8,6)
		p0.close!
		p0.offset!(0.125)

		assert_equal(p0,job.workpieces[0].paths[0])
		
	end
	
	def frag(str1,str2)
		return str2[str1.length..str2.length]
	end
	
	def test_header
		gen = Gcode_Generator.new(env)
		gen.header
	end
	
	def test_gen_group
		job = ex_job
		wp = job.workpieces[0]
		gen = Gcode_Generator.new(env)
		gen.header
		f1 = gen.gcode
		gen.initialize_group(wp.group)
		f2 = gen.gcode
		assert_equal("\n\n   (---------- WORKPIECE 1 ----------)\n   (---------- WORKPIECE NAME: G1 ----------)\n   (Group x-y bounds: [6.000\",4.000\"],[8.000\",6.000\"])\n   (Group height: 1.000\")\n   (Group base z: 0.000\")",frag(f1,f2))
		gen.cut_path_stack(wp.paths[0])
		f3 = gen.gcode
	end
	
	def test_gen_layer
		
	end
	
	def test_z_drop
		env1 = env
		env.feed_rate = 18
		env.vertical_feed_rate = 3
		
		#Null test...should have a feed rate of 18 if we go much further in the
		#horizontal direction than is limited by our vertical drop speed.
		gen = Gcode_Generator.new(env)
		gen.current_position = Geom::Point3d.new(0,0,1)
		gen.move_to(Geom::Point3d.new(20,0,0),:feed)
		assert_equal("\nG01 X20.0000 Y0.0000 Z0.0000 F18.0000",gen.gcode)
		
		#Equal test...descending 1 inch at 3 in/min will take 20 seconds.
		#20 seconds at 18 in/min (0.3 in/sec) = 6 in
		gen = Gcode_Generator.new(env)
		gen.current_position = Geom::Point3d.new(0,0,1)
		gen.move_to(Geom::Point3d.new(6,0,0),:feed)
		assert_equal("\nG01 X6.0000 Y0.0000 Z0.0000 F18.0000",gen.gcode)
		
		#Half test...same time (20 sec)
		#If we only go 3 inches, we have to take a total of 20 seconds.
		# 3/20 in/sec = 9 in/min
		gen = Gcode_Generator.new(env)
		gen.current_position = Geom::Point3d.new(0,0,1)
		gen.move_to(Geom::Point3d.new(3,0,0),:feed)
		assert_equal("\nG01 X3.0000 Y0.0000 Z0.0000 F9.0000",gen.gcode)
		
		#1/6 test...same time (20 sec)
		# 1/20 in/sec = 3 in/min
		gen = Gcode_Generator.new(env)
		gen.current_position = Geom::Point3d.new(0,0,1)
		gen.move_to(Geom::Point3d.new(1,0,0),:feed)
		assert_equal("\nG01 X1.0000 Y0.0000 Z0.0000 F3.0000",gen.gcode)
	end
end
