
load 'gpath/path_geom_lib.rb'
load 'gpath/pgl_segmentdef.rb'
load 'gpath/pgl_arcdef.rb'
load 'gpath/pgl_path.rb'
include Grays42_GPath
require 'test/unit'

$debug = false
$debug_num = nil

class PGL_TESTS < Test::Unit::TestCase

	def ents
		return Sketchup.active_model.entities
	end

	def pt(x,y)
		return Geom::Point3d.new(x,y,0)
	end
	
	def vec(x,y)
		return Geom::Vector3d.new(x,y,0)
	end
	
	def seg(x1,y1,x2,y2)
		return PGL_SegmentDef.new(pt(x1,y1),pt(x2,y2))
	end
	
	def to_rad(num)
		return num.to_f*Math::PI/180.0
	end
	
	def teardown
		$debug = false
	end

	def test_segments
		
		# --- Single intersect between two different lines ---
		
		@seg1 = seg(0,0,0,2)
		@seg1a = seg(0,2,2,2)
		assert_equal(pt(0,2), @seg1.intersect(@seg1a), "Single joint point")
		
		@seg1b = seg(-1,0,1,-1)
		assert_equal(nil, @seg1.intersect(@seg1b), "Non-intersecting different lines")
		
		@seg1c = seg(-1,1,1,1)
		assert_equal(pt(0,1), @seg1.intersect(@seg1c), "Mid-intersecting lines")
		
		
		# --- Vertical same-line segment intersects ---
		
		@seg3a = seg(0,1,0,3)
		assert_equal(pt(0,2), @seg1.intersect(@seg3a))
		
		@seg3b = seg(0,2,0,4)
		assert_equal(pt(0,2), @seg1.intersect(@seg3b))

		assert_equal(pt(0,0), @seg1.intersect(@seg1))
		
		# -- Horizontal same-line intersects
		
		@seg5 = seg(0,0,2,0)
		@seg5a = seg(1,0,3,0)
		assert_equal(pt(2,0), @seg5.intersect(@seg5a))
		
		@seg5b = seg(2,0,4,0)
		assert_equal(pt(2,0), @seg5.intersect(@seg5b))
		
		assert_equal(pt(0,0), @seg5.intersect(@seg5))
		
		@seg5c = seg(3,0,5,0)
		assert_equal(nil, @seg5.intersect(@seg5c))
		
		assert_equal(1,@seg5c.distance_from(pt(2,0)))
		assert_equal(0,@seg5c.distance_from(pt(3,0)))
		assert_equal(0,@seg5c.distance_from(pt(4,0)))
		assert_equal(0,@seg5c.distance_from(pt(5,0)))
		assert_equal(1,@seg5c.distance_from(pt(6,0)))
		
		assert_equal(Math::sqrt(2),@seg5c.distance_from(pt(2,1)))
		assert_equal(1,@seg5c.distance_from(pt(3,1)))
		assert_equal(1,@seg5c.distance_from(pt(4,1)))
		assert_equal(1,@seg5c.distance_from(pt(5,1)))
		assert_equal(Math::sqrt(2),@seg5c.distance_from(pt(6,1)))
		
		# Fuse testing
		
		@seg1 = seg(0,0,0,1)
		@seg2 = seg(0,1,0,2)
		
		@seg1.fuse!(@seg2)
		
		assert_equal(2,@seg1.length)
		assert_equal(0,@seg2.length)
		assert(@seg2.zero?)
		assert_equal(@seg1.end_pt,pt(0,2))
		
		@seg3 = seg(0,1,0,3)
		
		@seg1.fuse!(@seg3)
		assert_equal(3,@seg1.length)
		assert_equal(0,@seg3.length)
		assert(@seg3.zero?)
		assert_equal(@seg1.end_pt,pt(0,3))
		
	end
	
	def test_offset
		
		@org = Geom::Point3d.new(0,0,0)
		@vec_right = Geom::Vector3d.new(1,0,0)
		@vec_left = Geom::Vector3d.new(-1,0,0)
		@vec_up = Geom::Vector3d.new(0,1,0)
		
		@arc = PGL_ArcDef.new(@org,1,@vec_right,@vec_up)
		assert_equal(1,@arc.radius)
		assert_equal(@vec_right,@arc.vectors[0])
		assert_equal(@vec_up,@arc.vectors[1])
		
		@arc.offset!(-2)
		assert_equal(3,@arc.radius)
		
		@arc.reverse!
		
		@arc.offset!(-3)
		assert(@arc.length == 0)
		
		@seg_right = seg(0,0,1,0)
		@seg_right.offset!(1)
		assert_equal(seg(0,1,1,1),@seg_right)
		
		@seg_up = seg(0,0,0,1)
		@seg_up.offset!(-1)
		assert_equal(seg(1,0,1,1),@seg_up)
	end
	
	def test_tangent_vector
		
		@org = Geom::Point3d.new(0,0,0)
		@vec_right = Geom::Vector3d.new(1,0,0)
		@vec_left = Geom::Vector3d.new(-1,0,0)
		@vec_up = Geom::Vector3d.new(0,1,0)
		@vec_down = Geom::Vector3d.new(0,-1,0)
		
		@seg_up = seg(0,0,0,1)
		assert_equal(vec(0,1),@seg_up.tangent(0))
		assert_equal(vec(0,1),@seg_up.tangent(1))
		
		@arc = PGL_ArcDef.new(@org,2,@vec_right,@vec_up)
		assert_equal(@vec_up,@arc.tangent(0))
		assert_equal(@vec_left,@arc.tangent(1))
		@arc.reverse!
		assert_equal(@vec_right,@arc.tangent(0))
		assert_equal(@vec_down,@arc.tangent(1))
		
	end
	
	def test_arc_sweep
		
		@org = Geom::Point3d.new(0,0,0)
		@vec1 = Geom::Vector3d.new(1,0,0)
		@vec2 = Geom::Vector3d.new(-1,0,0)
		@vec3 = Geom::Vector3d.new(0,1,0)
		
		@arc = PGL_ArcDef.new(@org,1,@vec1,@vec1)
		assert_equal(to_rad(360),@arc.sweep)
		
		@arc4 = PGL_ArcDef.new(@org,1,@vec1,@vec3)
		assert_equal(to_rad(90),@arc4.sweep)
		@arc4.reverse!
		assert_equal(to_rad(90),@arc4.sweep)
		assert_equal(to_rad(135),@arc4.angular_distance_from(pt(-1,-1)))
		assert_equal(to_rad(90),@arc4.angular_distance_from(pt(0,-1)))
		assert_equal(to_rad(90),@arc4.angular_distance_from(pt(-1,0)))
		assert_equal(to_rad(45),@arc4.angular_distance_from(pt(1,-1)))
		assert_equal(to_rad(45),@arc4.angular_distance_from(pt(-2,2)))
		
		
		
		@arc2 = PGL_ArcDef.new(@org,1,@vec1,@vec2)
		assert_equal(to_rad(180),@arc2.sweep)
		@arc2 = PGL_ArcDef.new(@org,1,@vec2,@vec1)
		assert_equal(to_rad(180),@arc2.sweep)
		@arc2.reverse!
		assert_equal(to_rad(180),@arc2.sweep)
		
		
		@arc3 = PGL_ArcDef.new(@org,1,@vec3,@vec1)
		assert_equal(to_rad(270),@arc3.sweep)
		@arc3.reverse!
		assert_equal(to_rad(270),@arc3.sweep)
		
		assert_equal(0,@arc3.angular_distance_from(pt(0,0)))
		assert_equal(0,@arc3.angular_distance_from(pt(-1,0)))
		assert_equal(0,@arc3.angular_distance_from(pt(-2,-2)))
		assert_equal(0,@arc3.angular_distance_from(pt(0,-1)))
		assert_equal(to_rad(45),@arc3.angular_distance_from(pt(1,1)))
	end
	
	def test_circle_circle
		
		@up_norm = Geom::Vector3d.new(0,0,1)
		@down_norm = Geom::Vector3d.new(0,0,-1)
		
		@vec1 = Geom::Vector3d.new(1,0,0)
		
		@circ1 = PGL_ArcDef.new(Geom::Point3d.new(0,0,0),1,@vec1,@vec1)
		@circ2 = PGL_ArcDef.new(Geom::Point3d.new(0,3,0),2,@vec1,@vec1)
		@circ3 = PGL_ArcDef.new(Geom::Point3d.new(0,3,0),1,@vec1,@vec1)
		
		@int1 = @circ1.intersect(@circ2)
		assert_equal(Geom::Point3d.new(0,1,0), @int1)
		
		@int2 = @circ1.intersect(@circ3)
		@int3 = @circ2.intersect(@circ3)
		assert_equal(nil,@int2)
		assert_equal(nil,@int3)
		
		#Given a circle of radius 5, a triangle is made with sides 3 and 4 units long. Used to get some fixed points that are easy to test.
		@circ4 = PGL_ArcDef.new(Geom::Point3d.new(0,0,0),5,@vec1,@vec1)
		@circ5 = PGL_ArcDef.new(Geom::Point3d.new(0,6,0),5,@vec1,@vec1)
		
		#Executive change 3/28/14: all geom intersections should only pop 1 intersect point, this no longer works correctly.
		#I cannot envision a situation where a two-intersect between PGLs would be needed. Will revisit if necessary. May require a lot of refactoring.
		return
		@int4 = PGL_Intersect.new([Geom::Point3d.new(4,3,0),Geom::Point3d.new(-4,3,0)])
		assert_equal(@int4,@circ4.intersect(@circ5))
		
	end
	
	def test_arc_arc
		
		@vec_up = Geom::Vector3d.new(0,1,0)
		@vec_right = Geom::Vector3d.new(1,0,0)
		@vec_left = Geom::Vector3d.new(-1,0,0)
		
		@arc_right_to_top = PGL_ArcDef.new(Geom::Point3d.new(0,0,0),1,@vec_right,@vec_up)
		@arc_top_to_right = @arc_right_to_top.reverse
		@arc_top_to_left = PGL_ArcDef.new(Geom::Point3d.new(2,0,0),1,@vec_up,@vec_left)
		@arc_left_to_top = @arc_top_to_left.reverse
		@int = @arc_top_to_right.intersect(@arc_top_to_left)
		
		assert_equal(pt(1,0),@int)
		
		@arc_top_to_right.radius = 5
		@arc_top_to_left.radius = 5
		@arc_top_to_left.center = Geom::Point3d.new(6,0,0)
		@int = @arc_top_to_right.intersect(@arc_top_to_left)
		
		assert_equal(pt(3,4),@int)

		@arc_left_to_top = @arc_top_to_left.reverse
		@int = @arc_top_to_right.intersect(@arc_left_to_top)
		assert_equal(pt(3,4),@int)
		
		@arc_top_to_slightly_right = @arc_top_to_right.clone
		@arc_top_to_slightly_right.set_vector(1,Geom::Vector3d.new(3,4.1,0))
		@arc_top_to_slightly_right.separation_allowed = 0.3
		@int = @arc_top_to_slightly_right.intersect(@arc_top_to_left)
		
		assert_equal(Geom::Point3d.new(3,4,0),@int)
		
		@arc_right_side_circle = @arc_top_to_left.clone
		@arc_top_to_left.set_vector(1,@vec_up)
		@int = @arc_top_to_slightly_right.intersect(@arc_right_side_circle)
		
		assert_equal(Geom::Point3d.new(3,4,0),@int)
		
		#Test fuses
		
		@down = Geom::Vector3d.new(0,0,-1)
		@arc1 = PGL_ArcDef.new(pt(0,0),1,vec(0,1),vec(1,1),@down)
		@arc2 = PGL_ArcDef.new(pt(0,0),1,vec(1,1),vec(1,0),@down)
		
		@arc1.fuse!(@arc2)
		
		assert_equal(vec(1,0),@arc1.vectors[1])
		assert_equal(Math::PI/2.0,@arc1.length)
		assert(@arc2.zero?)
		
	end
	
	def test_arc_line
		
		@vec_up = Geom::Vector3d.new(0,1,0)
		@vec_right = Geom::Vector3d.new(1,0,0)
		@vec_left = Geom::Vector3d.new(-1,0,0)
		
		@arc_right_to_top = PGL_ArcDef.new(Geom::Point3d.new(0,0,0),5,@vec_right,@vec_up)
		@arc_top_to_right = @arc_right_to_top.reverse
		@arc_top_to_left = PGL_ArcDef.new(Geom::Point3d.new(10,0,0),5,@vec_up,@vec_left)
		@int = @arc_top_to_right.intersect(@arc_top_to_left)
		
		@seg1 = seg(0,0,0,2)
		
		
	end
	
	def test_path
		
		@p = PGL_Path.new
		assert_equal(false,@p.closed?)
		assert_equal(0,@p.array_length)
		assert_equal(0,@p.length)
		
		@seg1 = seg(0,0,0,2)
		@p.append_item(@seg1)
		
		assert_equal(1,@p.array_length)
		assert_equal(2,@p.length)
		
		@seg2 = seg(0,2,2,2)
		@p.append_item(@seg2)
		
		assert_equal(2,@p.array_length)
		assert_equal(4,@p.length)
		
		@seg3 = seg(2,2,4,2)
		@p.append_item(@seg3)
		
		assert_equal(2,@p.array_length)
		assert_equal(6,@p.length)
		
		halfpi = Math::PI/2.0
		tot = 6 + 2*halfpi
		
		#testing imperfect join
		@arc1 = PGL_ArcDef.new(pt(4,4),2,vec(0.1,-1),vec(1,0),Geom::Vector3d.new(0,0,1),0.3)
		@p.append_item(@arc1)
		
		assert_equal(3,@p.array_length)
		assert_equal(tot,@p.length)
		
		@p.verify!
		@p.reverse!
		
		assert_equal(3,@p.array_length)
		assert_equal(tot,@p.length)
		assert_equal(@arc1,@p.pgl_items.first)
		
		assert_equal([vec(1,0),vec(0,-1)],@p.pgl_items.first.vectors)
		
		@p.verify!
		
		#Offset testing
		
		assert_equal(2,@p.pgl_items.first.radius)
		@p.offset!(1)
		assert_equal(3,@p.array_length)
		assert_equal([vec(1,0),vec(0,-1)],@p.pgl_items.first.vectors)
		assert_equal(3,@p.pgl_items.first.radius)
		@p.verify!
		
	end
	
	def test_small_path_offset
		@path1 = PGL_Path.new
		@path1 << seg(0,0,0,1)
		@path1 << seg(0,1,1,1)
		
		@path2 = PGL_Path.new
		@path2 << seg(-1,0,-1,1)
		@path2 << PGL_ArcDef.new(pt(0,1),1,vec(-1,0),vec(0,1),Geom::Vector3d.new(0,0,-1))
		@path2 << seg(0,2,1,2)
	
		@path1o = @path1.offset(1)
		assert_equal(@path1o.pgl_items[0],@path2.pgl_items[0])
		assert_equal(@path1o.pgl_items[1],@path2.pgl_items[1])
		assert_equal(@path1o.pgl_items[2],@path2.pgl_items[2])
		assert_equal(@path1o,@path2)
		off = @path1.offset(-1)
		assert(off.zero?)
		assert(0,off.array_length)
		
		@path1.reverse!
		@path2.reverse!
		@path1o = @path1.offset(-1)
		assert(@path1.offset(1).zero?)
		assert(0,@path1.offset(1).array_length)
		assert_equal(@path1o,@path2)
	end
	
	def test_weird_square_path
		@path1 = PGL_Path.new
		@path1 << seg(0,-1,0,0)
		@path1 << seg(0,0,1,0)
		@path1 << seg(1,0,1,2)
		@path1 << seg(1,2,0,2)
		@path1 << seg(0,2,0,3)
		
		@path2 = PGL_Path.new
		@path2 << seg(-1,-1,-1,0)
		@path2 << PGL_ArcDef.new(pt(0,0),1,vec(-1,0),vec(0,1),Geom::Vector3d.new(0,0,-1))
		@path2 << PGL_ArcDef.new(pt(0,2),1,vec(0,-1),vec(-1,0),Geom::Vector3d.new(0,0,-1))
		@path2 << seg(-1,2,-1,3)

		@path1o = @path1.offset(1)
		
		
		assert_equal(@path2.array_length,@path1o.array_length)
		assert_equal(@path2.length,@path1o.length)
		assert_equal(@path1.offset(1),@path2)
		
	end
	
	def test_complete_square
		path1 = PGL_Path.new
		path1 << seg(1,1,1,3)
		path1 << seg(1,3,3,3)
		path1 << seg(3,3,3,1)
		assert(path1.closed? == false)
		path1 << seg(3,1,1,1)
		
		assert_equal(false,path1.closed?)
		path1.close!
		assert(path1.closed?)
		assert_equal(4,path1.array_length)
		assert_equal(8,path1.length)
		
		path2 = PGL_Path.new
		path2 << seg(0,1,0,3)
		path2 << PGL_ArcDef.new(pt(1,3),1,vec(-1,0),vec(0,1),Geom::Vector3d.new(0,0,-1))
		path2 << seg(1,4,3,4)
		path2 << PGL_ArcDef.new(pt(3,3),1,vec(0,1),vec(1,0),Geom::Vector3d.new(0,0,-1))
		path2 << seg(4,3,4,1)
		path2 << PGL_ArcDef.new(pt(3,1),1,vec(1,0),vec(0,-1),Geom::Vector3d.new(0,0,-1))
		path2 << seg(3,0,1,0)
		path2 << PGL_ArcDef.new(pt(1,1),1,vec(0,-1),vec(-1,0),Geom::Vector3d.new(0,0,-1))
		
		path1o = path1.offset(1)
		assert_equal(path2,path1o)
	end
	
	def test_circle
		path1 = PGL_Path.new
		path1 << PGL_ArcDef.new(pt(1,3),1,vec(-1,0),vec(0,1),Geom::Vector3d.new(0,0,-1))
		path1 << PGL_ArcDef.new(pt(1,3),1,vec(0,1),vec(1,0),Geom::Vector3d.new(0,0,-1))
		path1 << PGL_ArcDef.new(pt(1,3),1,vec(1,0),vec(0,-1),Geom::Vector3d.new(0,0,-1))
		
		assert_equal(1,path1.array_length)
		
		path1 << PGL_ArcDef.new(pt(1,3),1,vec(0,-1),vec(-1,0),Geom::Vector3d.new(0,0,-1))
		assert_equal(1,path1.array_length)
		assert_equal(false,path1.pgl_items[0].zero?)
		path1.close!
		assert(path1.closed?)
		
		path1.reverse!
		assert_equal(1,path1.array_length)
		assert_equal(false,path1.pgl_items[0].zero?)
		assert(path1.closed?)
		path1.reverse!
		
		assert_equal(1,path1.pgl_items[0].radius)
		path1.offset!(1)
		assert_equal(1,path1.array_length)
		assert_equal(2,path1.pgl_items[0].radius)
		path1.offset!(-2)
		assert(0,path1.array_length)
		assert(path1.zero?)
	end
	
	def test_path_starts_with_zero
		
		path1 = PGL_Path.new
		path1 << seg(1,1,1,2)
		path1 << seg(1,2,3,2)
		
		path1.offset!(-1)
		
		assert_equal(1,path1.length)
		assert_equal(1,path1.array_length)
		assert_equal(pt(2,1),path1.start_pt)
	end
	
	def test_zero
		seg1 = seg(1,1,1,2)
		int = seg1.intersect(seg(1,1,1,1))
		assert_equal(pt(1,1),int)
		
		seg2 = seg(1,0,1,0)
		assert(seg2.zero?)
		assert_equal(seg2.start_pt,pt(1,0))
		assert_equal(seg2.end_pt,pt(1,0))
		
		arc1 = PGL_ArcDef.new(pt(0,0),1,vec(1,0),vec(1,0),Geom::Vector3d.new(0,0,-1))
		arc1.zero!
		assert_equal(arc1.start_pt,pt(1,0))
		assert_equal(arc1.end_pt,pt(1,0))
		
		int = seg2.intersect(arc1)
		assert_equal(pt(1,0),int)
	end
	
	def test_split
		a = seg(0,0,0,2)
		b = a.split(pt(0,1))
		assert_equal(a,seg(0,0,0,1))
		assert_equal(b,seg(0,1,0,2))
		
		a = PGL_ArcDef.new(pt(0,0),1,vec(1,0),vec(-1,0))
		b = a.split(pt(0,1))
		assert_equal(a,PGL_ArcDef.new(pt(0,0),1,vec(1,0),vec(0,1)))
		assert_equal(b,PGL_ArcDef.new(pt(0,0),1,vec(0,1),vec(-1,0)))
	end
	
	def test_subpaths
		path = PGL_Path.new
		path << seg(0,0,0,1)
		path << seg(0,1,2,-1)
		path << seg(2,-1,4,1)
		path << seg(4,1,4,-1)
		path << seg(4,-1,2,1)
		path << seg(2,1,0,-1)
		path << seg(0,-1,0,0)
		
		
	end
	
	def test_fracture
		path = PGL_Path.new
		path << seg(0,0,0,2)
		path << seg(0,2,2,0)
		path << seg(2,0,2,2)
		path << seg(2,2,0,0)
		
		path_mended = path.clone
		
		path_fractured = []
		path_fractured << seg(0,0,0,2)
		path_fractured << seg(0,2,1,1)
		path_fractured << seg(1,1,2,0)
		path_fractured << seg(2,0,2,2)
		path_fractured << seg(2,2,1,1)
		path_fractured << seg(1,1,0,0)
		
		assert_equal(path_mended,path)
		
		path.fracture!
		
		assert_equal(path_fractured,path.pgl_items)
		
		path.mend!
		
		assert_equal(path_mended,path)
		
	end
	
	def test_interior_loops
		
		#Fully functional test but doesn't test what I think it does so I'm ignoring it, will use it later if needed
	
		path = PGL_Path.new
		path << seg(0,0,0,4)
		path << seg(0,4,1.5,4)
		path << seg(1.5,4,1,2)
		path << seg(1,2,3,2)
		path << seg(3,2,2.5,4)
		path << seg(2.5,4,4,4)
		path << seg(4,4,4,0)
		path << seg(4,0,0,0)
		path.close!
		
		#path.draw
	end
end
